#!/usr/bin/python3
"""
Created: June 2020
Author: amanno
Description: A File to hold the valid device parameters per environment.  Each environment (local, dev01 and test03)
  has been loaded with the sensor-sim objects such that the information in the databases will allow the sensors
  to authenticate with the data provided in the "<env>_valid_auth_params" objects.  This file also holds constants
  for common String values and the list objects used to fuzz values.
"""
import os
import random
from test_data import get_org, get_devices, get_policy


class Strings:
    """
    Common string values that very rarely every change and are used throughout the code.
    """
    REG_ID = 'Registration-ID'
    GUID = 'Device-GUID'
    DEV_ENV = 'Device-Environment'
    DEV_CONTEXT = 'Device-Context'
    APP_JSON = 'application/json'
    APP_PROTO = 'application/x-protobuf'
    ACCEPT = 'Accept'
    CACHE_HIT = 'Cache-Hit'
    CONTENT_TYPE = 'Content-Type'


class Lists:
    DEVICE_TYPES_NUMS = [0, 1, 2, 3, 4, 5, 10]
    DEVICE_TYPES_NAMES = ['UNKNOWN', 'WINDOWS', 'ANDROID', 'MAC', 'IOS', 'LINUX', 'OTHER']
    DEPLOYMENT_TYPES_NUMS = [0, 1, 2, 3, 4, 5]
    DEPLOYMENT_TYPES_NAMES = ['ENDPOINT', 'WORKLOAD', 'VDI', 'AWS', 'AZURE', 'GCP', '']


class FuzzList:
    """
    List of values to fuzz, simply append to these list to add more fuzzers.
    """
    BAD_ORG_KEYS = ['R76KWHRX', None, b'101', 'X' * 32]
    BAD_REG_IDS = ['A', 'B', '=', None, b'101', 'X' * 32]
    BAD_GUIDS = [None, 'X' * 65]


# A valid parameter is a tuple defined as: (<Registration-ID>, <Device-GUID>, <Org-Key>)
# These are pre-loaded into the DB when the mech is started
local_valid_device_group_ids = [(1, 'R76KWHRN'), (3, 'R76KWHRN'), (8, 'K7MGDHQ7')]

local_deregistered_device_params = []

local_valid_auth_params = [('1-1', '498d862e93c856dfd6e72c1af0ab8a438c66fc91b4b22f6dbd60d18f1f3ef85f', 'R76KWHRN'),
                           ('1-2', '39aa3ac41f197403494f6636abac97ba5e273831ed75efc6406e6f9f53616e16', 'R76KWHRN'),
                           ('1-3', '9df236abde21ffcc92a60d7d378cf383faefd88f20c8380e37c8a6955aa53855', 'R76KWHRN'),
                           ('1-4', '824a0e3fa100dca462bc8ca63cd17e7a05aa6dc51fbab07d4922f88b2b065d2e', 'R76KWHRN'),
                           ('1-5', '25079ede03124c5e50b71474fc4d3fa4fb64e5c9ca7071f7edbd6821b7e4b221', 'R76KWHRN'),
                           ('1-6', 'a9967487a3ab12a7986e2567be773d6bb3e75ae0d815f71709af72fa663c406a', 'R76KWHRN'),
                           ('2-7', 'a9967487a3ab12a7986e2567be773d6bb3e75ae0d815f71709af72fa663c406b', 'K7MGDHQ7'),
                           ('2-8', 'a9967487a3ab12a7986e2567be773d6bb3e75ae0d815f71709af72fa663c406c', 'K7MGDHQ7'),
                           ('1-9', 'a9967487a3ab12a7986e2567be773d6bb3e75ae0d815f71709af72fa663c406d', 'R76KWHRN')]


# Dev01 Sensors - Fox, Link, Samus, Kirby, Mario, Pikachu
# Org Name: synergy-ems-pipeline (ID: 1661788)
dev01_valid_device_group_ids = [(6277889, '968T9DWZQ')]

dev01_deregistered_device_params = [('1661788-6072056', '48d1263634174e7fa50f2c5d7572f90c', '968T9DWZQ', Strings.APP_JSON),
                                    ('1661788-6072056', '48d1263634174e7fa50f2c5d7572f90c', '968T9DWZQ', Strings.APP_PROTO)]

dev01_valid_auth_params = [('1661788-6065860', 'c93ffe280f5f4587a5959187e2793ed9', '968T9DWZQ'),
                           ('1661788-6065904', '345783b7b2fe4a3ab2056f81bbbd3def', '968T9DWZQ'),
                           ('1661788-6065940', '7429d10c41464b149c41d8452e1af645', '968T9DWZQ'),
                           ('1661788-6065958', '77068a1a5ecb4b12ba7e1dc0b79b766c', '968T9DWZQ'),
                           ('1661788-6065979', '4c4aba75a350428a8c13ed243c060209', '968T9DWZQ'),
                           ('1661788-6065996', '684a16aa854f4b4bbe876a33f8f836ec', '968T9DWZQ')]


# Test03 Sensors - IronMan, Hulk, Thor, Jarvis, Hawkeye, CaptainAmerica
# Org Name: synergy-ems-pipeline (ID: 1558879)
test03_valid_device_group_ids = [(6145407, 'XYGTRG8JE')]

test03_deregistered_device_params = [('1558879-11046623', 'acd462b539cd4f1288369bd56c639027', 'XYGTRG8JE', Strings.APP_JSON),
                                     ('1558879-11046623', 'acd462b539cd4f1288369bd56c639027', 'XYGTRG8JE', Strings.APP_PROTO)]

test03_valid_auth_params = [('1558879-10964238', '3825d8d6028e4adab5cefe71857f4d2c', 'XYGTRG8JE'),
                            ('1558879-10964287', '4a403a514a0b4426b07bad8c9d0b5574', 'XYGTRG8JE'),
                            ('1558879-10964310', 'bbf3f2b4811c4ea8ac4f93c1ca6bc39e', 'XYGTRG8JE'),
                            ('1558879-10964325', '478aca88dbe84be5a3b4b2e2af47739f', 'XYGTRG8JE'),
                            ('1558879-10964344', '96527ec226c84aaca010c095e6745014', 'XYGTRG8JE'),
                            ('1558879-10964374', 'a0dbe31ccce846aebab942fbcd397240', 'XYGTRG8JE')]

# Special case that isn't constants but actually inserts new test data
# Once we're off MCS1 we can delete the hard-coded stuff above and just use dynamically generated data for testing
if os.environ.get('POSTGRES_ENV', None) == "cb-emd1":
    # Special case for partial. Not quite constants, but we can remove the constant once MCS1 is deprecated
    org = get_org()
    devices = get_devices(org['org_key'], 6, False)
    # These tests consider a 401, which is returned when a device doesn't belong to a device group, a dereg'd sensor
    deregistered_devices = get_devices(org['org_key'], 3, True)
    device_group = get_policy(devices, org['org_id'])

    emd1_valid_device_group_ids = [(device_group['device_group_id'], org['org_key'])]

    emd1_deregistered_device_params = []
    for device in deregistered_devices:
        emd1_deregistered_device_params.append((f"{org['org_id']}-{device['device_id']}", device['device_guid'], org['org_key'], Strings.APP_JSON if random.randint(0,1) == 0 else Strings.APP_PROTO))

    emd1_valid_auth_params = []
    for device in devices:
        emd1_valid_auth_params.append((f"{org['org_id']}-{device['device_id']}", device['device_guid'], org['org_key']))
