#!/usr/bin/python3
"""
Created: June 2020
Author: amanno
Description: A test to verify calls to evict cache return expected status codes.
NOTE: For security reasons if device-ids are not found in the cache a 204 response is still returned.
NOTE: Requests that fail to authenticate are NOT cached.
"""
import pytest
import requests


class TestCache204:

    @pytest.mark.parametrize('device_id, org_key', pytest.valid_cache_device_id_params)
    def test_evict_device(self, supply_main_url: str, device_id: int, org_key: str):
        """
        Successful evictions on device-ids should return a 204 status code.
        :param supply_main_url: Base cache URL
        :param device_id: Device-Id to evict
        :param org_key: Org-key of the device to evict
        :return: None
        """
        url = f'{supply_main_url}/{org_key}/device/{device_id}'
        cache_resp = requests.delete(url, headers={"Prefer": "respond-sync"})
        assert cache_resp.status_code == 204, f'Did not successfully evict device: {device_id}'

    @pytest.mark.parametrize('org_id, org_key', pytest.valid_cache_org_id_params)
    def test_evict_org(self, supply_main_url: str, org_id: int, org_key: str):
        """
        Successful evictions on orgs should return a 204 status code
        :param supply_main_url: Base cache URL
        :param org_id: Org-Id to evict
        :param org_key: Org-Key of the org to evict
        :return: None
        """
        url = f'{supply_main_url}/{org_key}/org/{org_id}'
        cache_resp = requests.delete(url, headers={"Prefer": "respond-sync"})
        assert cache_resp.status_code == 204, f'Did not successfully evict {org_key}'

    @pytest.mark.parametrize('device_group_id, org_key', pytest.valid_device_group_ids)
    def test_evict_device_group(self, supply_main_url: str, device_group_id: int, org_key: str):
        """
        Successful evictions of the device_group_id should return a 204 status code.
        NOTE: This eviction call should also evict any devices in the given device_group
        :param supply_main_url: Base cache URL
        :param device_group_id: Device Group ID to evict
        :param org_key: Org-key of device group
        :return: None
        """
        url = f'{supply_main_url}/{org_key}/device-group/{device_group_id}'
        cache_resp = requests.delete(url, headers={"Prefer": "respond-sync"})
        assert cache_resp.status_code == 204, f'Did not successfully evict device group {device_group_id}'

    @pytest.mark.parametrize('device_id, org_key', pytest.cache_id_not_found_params)
    def test_evict_device_id_not_found(self, supply_main_url: str, device_id: int, org_key: str):
        """
        Verify there is no indication whether a device-id existed in the cache or not.
        :param supply_main_url: Base cache URL
        :param device_id: Device-ID not found in the cache
        :param org_key: Org-Key of device-id
        :return: None
        """
        url = f'{supply_main_url}/{org_key}/device/{device_id}'
        cache_resp = requests.delete(url, headers={"Prefer": "respond-sync"})
        assert cache_resp.status_code == 204, 'Device Ids not found should return a 204 only'


class TestCache400:

    @pytest.mark.parametrize('device_id, org_key', pytest.bad_cache_id_params)
    def test_evict_malformed_device_id(self, supply_main_url: str, device_id: int, org_key: str):
        """
        Verify a Bad Request is returned when fuzzed values are entered for the device-id
        :param supply_main_url: Base cache URL
        :param device_id: Fuzzed Device-ID
        :param org_key: Org-key of device
        :return: None
        """
        url = f'{supply_main_url}/{org_key}/device/{device_id}'
        cache_resp = requests.delete(url)
        assert cache_resp.status_code == 400, 'Malformed Device IDs should return a 400 - Bad Request'

    @pytest.mark.parametrize('org_id, org_key', pytest.bad_cache_id_params)
    def test_evict_malformed_org_id(self, supply_main_url: str, org_id: int, org_key: str):
        """
        Verify a Bad Request is returned when fuzzed values are entered for the device-id
        :param supply_main_url: Base cache URL
        :param org_id: Fuzzed Org-ID
        :param org_key: Org-Key of org-id
        :return: None
        """
        url = f'{supply_main_url}/{org_key}/org/{org_id}'
        cache_resp = requests.delete(url)
        assert cache_resp.status_code == 400, 'Malformed Org Ids/Keys should return a 400 - Bad Request'

    @pytest.mark.parametrize('device_group_id, org_key', pytest.bad_cache_id_params)
    def test_evict_malformed_device_group_id(self, supply_main_url: str, device_group_id: int, org_key: str):
        """
        Verify a Bad Request is returned when fuzzed values are entered for the device-group-id
        :param supply_main_url: Base cache URL
        :param device_group_id: Fuzzed device-group-id
        :param org_key: Org-Key of the device_group
        :return: None
        """
        url = f'{supply_main_url}/{org_key}/device-group/{device_group_id}'
        cache_resp = requests.delete(url)
        assert cache_resp.status_code == 400, 'Malformed Device Group Id should return a 400 - Bad Request'
